pfUI:RegisterModule("managems", function ()
  -- TODO: update images
  -- TODO: two themes (icons/squares)
  local _, playerClass = UnitClass("player")
  if playerClass ~= "MAGE" then return end

  pfUI:LoadConfig_managems()
  pfUI:LoadGui_managems()

  local border = C.appearance.border.unitframes ~= "-1" and C.appearance.border.unitframes or C.appearance.border.default

  local managems_list = {
    "Mana Ruby",
    "Mana Citrine",
    "Mana Jade",
    "Mana Agate",
  }

  local managems_trained = {}


  local function FindItemInBags(item_name)
    for bag = 4, 0, -1 do
      for slot = 1, GetContainerNumSlots(bag) do
        local _, item_count = GetContainerItemInfo(bag, slot);
        if item_count then
          local item_link = GetContainerItemLink(bag,slot)
          local _, _, item_parse = strfind(item_link, "(%d+):")
          local query_name, _, _, _, _, _ = GetItemInfo(item_parse)
          if query_name and query_name ~= "" then
            if strupper(query_name) == strupper(item_name) then
              return bag, slot
            end
          end
        end
      end
    end
    return nil
  end

  local function GetTrainedGems()
    managems_trained = {}
    for i, n in ipairs(managems_list) do
      local IsGemTrained = libspell.GetSpellIndex("Conjure " .. n)
      if IsGemTrained ~= nil then
        table.insert(managems_trained, n)
      end
    end
    return counter
  end

  local function RefreshGemFrame(frame)
    if managems_trained[frame.index] ~= nil then
      -- set texture
      frame.gem_name = managems_trained[frame.index]
      frame.spell_name = "Conjure " .. managems_trained[frame.index]
      local spell_id, book_type = libspell.GetSpellIndex(frame.spell_name)
      frame.texture:SetTexture(.15,.15,.15,1)

      --set gem color
      local bagID, slotID = FindItemInBags(frame.gem_name)
      if bagID ~= nil then
        local start, duration, isEnabled = GetContainerItemCooldown(bagID, slotID)
        if (start > 0 and duration > 2) or pfUI.managems.cd_active ~= nil then
          pfUI.managems.cd_active = start + duration
          frame.texture:SetTexture(.6,.1,.1,1)
        else
          frame.texture:SetTexture(.1,.6,.1,1)

        end
      else
        frame.texture:SetTexture(.15,.15,.15,1)
      end

      frame:Show()
    else
      frame:Hide()
    end
    return
  end

  local function CreateGemFrame(index)
    local gem_frame = CreateFrame("Frame", "pfManaGemsGem" .. index, UIParent)

    gem_frame.index = index

    gem_frame:SetFrameStrata("MEDIUM")
    gem_frame:SetWidth(C.managems.size)
    gem_frame:SetHeight(C.managems.size)
    gem_frame:EnableMouse(true)
    gem_frame.texture = gem_frame:CreateTexture(nil, "OVERLAY")
    gem_frame.texture:SetAllPoints(gem_frame)

    CreateBackdrop(gem_frame)
    RefreshGemFrame(gem_frame)

    return gem_frame

  end

  pfUI.managems = CreateFrame("Frame", "pfManaGemsFrame", UIParent)
  pfUI.managems:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
  pfUI.managems:RegisterEvent("BAG_UPDATE_COOLDOWN")
  pfUI.managems:RegisterEvent("UNIT_INVENTORY_CHANGED")
  pfUI.managems:RegisterEvent("LEARNED_SPELL_IN_TAB")

  pfUI.managems.frames = {}
  for i=1, 4 do
    pfUI.managems.frames[i] = CreateGemFrame(i)
  end

  local function ManagemsRefresh()
    GetTrainedGems()
    for i=1, 4 do
      RefreshGemFrame(pfUI.managems.frames[i])
    end
  end

  pfUI.managems:SetScript("OnEvent", function()
    if event == "UNIT_INVENTORY_CHANGED" and arg1 ~= "player" then return end
    pfUI.api.QueueFunction(ManagemsRefresh)
  end)

  pfUI.managems:SetScript("OnUpdate", function()
    if (this.tick or 1) > GetTime() then return else this.tick = GetTime() + 0.5 end
    if this.cd_active ~= nil then
      if this.cd_active > GetTime() then return end
      this.cd_active = nil
      this:GetScript("OnEvent")()
    else
      return
    end
  end)

  function pfUI.managems:UpdateConfig()
    local spacing = 5
    local offset
    for i=1, 4 do
      pfUI.managems.frames[i]:SetWidth(C.managems.size)
      pfUI.managems.frames[i]:SetHeight(C.managems.size)
      if pfUI.uf.player ~= nil then
        spacing = (floor(pfUI.uf.player:GetHeight()) - 4 * tonumber(C.managems.size)) / 3
        offset = -(i - 1) * (tonumber(C.managems.size) + spacing)
        pfUI.managems.frames[i]:SetPoint("TOPLEFT", pfUI.uf.player, "TOPRIGHT", border * 3, offset)
      else
        pfUI.managems.frames[i]:SetPoint("CENTER", UIParent, "CENTER", offset, border * 3)
      end

      UpdateMovable(pfUI.managems.frames[i])
    end

    pfUI.managems:GetScript("OnEvent")()
  end

  pfUI.managems:UpdateConfig()

end)